==> git-sql-2.0.3-sources.jar: io/github/andreyzebin/gitSql/git2/VersionedFiles.java <==
package io.github.andreyzebin.gitSql.git2;

import io.github.andreyzebin.gitSql.FSOffsetProxy;
import io.github.andreyzebin.gitSql.FileSystem;
import io.github.andreyzebin.gitSql.bash.Bash;
import io.github.andreyzebin.gitSql.git.GitSource;
import io.github.andreyzebin.gitSql.git.GitVersions;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.Reader;
import java.io.Writer;
import java.nio.file.Files;
import java.nio.file.Path;
import java.util.Arrays;
import java.util.Comparator;
import java.util.Objects;
import java.util.function.Consumer;
import java.util.function.Function;
import lombok.extern.slf4j.Slf4j;

@Slf4j
public class VersionedFiles implements FileSystem {


    private final boolean isReadOnly;
    private final GitVersions source;
    private final TerminalIO term;
    private StringBuffer filesToAdd;

    public VersionedFiles(boolean isReadOnly, GitVersions source, TerminalIO term) {
        this.isReadOnly = isReadOnly;
        this.source = source;
        this.term = term;
        filesToAdd = new StringBuffer();
    }

    public GitVersions getSource() {
        return source;
    }

    @Override
    public Writer put(Path path) {
        try {
            Path newFile = getAbs(path);
            Files.createDirectories(newFile.getParent());
            Bash.append(filesToAdd, path.toString());

            return new FileWriter(newFile.toFile(), false);
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
    }

    private Path getAbs(Path path) {
        return getRoot().resolve(path);
    }

    @Override
    public boolean erase(Path path) {
        if (exists(path)) {
            term.eval("rm " + Bash.escape(Bash.posix(getAbs(path))));
            return true;
        }

        return false;

    }

    @Override
    public Writer patch(Path path) {
        try {
            Path newFile = getAbs(path);
            Files.createDirectories(newFile.getParent());
            Bash.append(filesToAdd, path.toString());

            return new FileWriter(newFile.toFile(), true);
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
    }

    @Override
    public Reader get(Path path) {
        try {
            return new FileReader(getAbs(path).toFile());
        } catch (FileNotFoundException e) {
            throw new RuntimeException(e);
        }
    }

    @Override
    public boolean exists(Path path) {
        String pwd = term.eval("pwd");
        try {
            term.eval("cd " + Bash.escape(Bash.posix(getRoot())));
            return Boolean.parseBoolean(
                    term.eval("FILE=" + Bash.escape(Bash.posix(path)) + System.lineSeparator()
                            + "if [ -f \"$FILE\" ]; then\n"
                            + "    echo true \n"
                            + "else \n"
                            + "    echo false \n"
                            + "fi")
            );
        } finally {
            term.eval("cd " + pwd);
        }
    }

    @Override
    public boolean isDir(Path path) {
        return getAbs(path).toFile().isDirectory();
    }

    @Override
    public int run(String cmd, Consumer<String> stdOut, Consumer<String> stdErr) {
        String pwd = term.eval("pwd");
        try {
            term.eval("cd " + Bash.escape(Bash.posix(getAbs(Path.of("")))));
            return term.exec(cmd, stdOut, stdErr);
        } finally {
            term.eval("cd " + pwd);
        }
    }

    @Override
    public void find(Function<Path, Boolean> needContinue, Consumer<Path> sayBye, Comparator<Path> sorting) {
        Path root = getRoot();
        FileSystem.tVerse(
                root,
                (p) -> needContinue.apply(root.relativize(p)),
                (p) -> sayBye.accept(root.relativize(p)),
                sorting
        );
    }

    @Override
    public void flush() {
        if (isReadOnly) {
            return;
        }

        if (!filesToAdd.toString().isBlank()) {
            String pwd = term.eval("pwd");
            try {
                term.eval("cd " + Bash.escape(Bash.posix(getRoot())));
                Arrays.stream(filesToAdd.toString().split(System.lineSeparator()))
                        .forEach(cFile -> GitBindings.add(term, Path.of(cFile)));
                if (GitBindings.hasStatus(term)) {
                    GitBindings.commit(term);
                    term.eval("cd ..");
                    source.flush();
                } else {
                    log.debug("Status has no changes, skip commit.");
                }
            } finally {
                term.eval("cd " + pwd);
            }
        }

        filesToAdd = new StringBuffer();
    }

    @Override
    public FileSystem cd(Path jump) {
        return new FSOffsetProxy(this, jump);
    }


    @Override
    public void close() throws Exception {
        source.close();
    }


    public Path getRoot() {
        return source.getRoot();
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) {
            return true;
        }
        if (o == null || getClass() != o.getClass()) {
            return false;
        }
        VersionedFiles gitFiles = (VersionedFiles) o;
        return Objects.equals(source, gitFiles.source);
    }

    @Override
    public int hashCode() {
        return Objects.hash(source);
    }
}

==> git-sql-2.1.2-sources.jar: io/github/andreyzebin/gitSql/git2/VersionedFiles.java <==
package io.github.andreyzebin.gitSql.git2;

import io.github.andreyzebin.gitSql.FSOffsetProxy;
import io.github.andreyzebin.gitSql.FileSystem;
import io.github.andreyzebin.gitSql.bash.Bash;
import io.github.andreyzebin.gitSql.git.GitVersions;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.Reader;
import java.io.Writer;
import java.nio.file.Files;
import java.nio.file.Path;
import java.util.Arrays;
import java.util.Comparator;
import java.util.Objects;
import java.util.function.Consumer;
import java.util.function.Function;
import lombok.extern.slf4j.Slf4j;

@Slf4j
public class VersionedFiles implements FileSystem {


    private final boolean isReadOnly;
    private final GitVersions source;
    private final TerminalIO term;
    private StringBuffer filesToAdd;

    public VersionedFiles(boolean isReadOnly, GitVersions source, TerminalIO term) {
        this.isReadOnly = isReadOnly;
        this.source = source;
        this.term = term;
        filesToAdd = new StringBuffer();
    }

    public GitVersions getSource() {
        return source;
    }

    @Override
    public Writer put(Path path) {
        try {
            Path newFile = getAbs(path);
            Files.createDirectories(newFile.getParent());
            Bash.append(filesToAdd, path.toString());

            return new FileWriter(newFile.toFile(), false);
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
    }

    private Path getAbs(Path path) {
        return getRoot().resolve(path);
    }

    @Override
    public boolean erase(Path path) {
        if (exists(path)) {
            term.eval("rm " + Bash.encode(getAbs(path)));
            return true;
        }

        return false;

    }

    @Override
    public Writer patch(Path path) {
        try {
            Path newFile = getAbs(path);
            Files.createDirectories(newFile.getParent());
            Bash.append(filesToAdd, path.toString());

            return new FileWriter(newFile.toFile(), true);
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
    }

    @Override
    public Reader get(Path path) {
        try {
            return new FileReader(getAbs(path).toFile());
        } catch (FileNotFoundException e) {
            throw new RuntimeException(e);
        }
    }

    @Override
    public boolean exists(Path path) {
        return Files.exists(getAbs(path));
    }

    @Override
    public boolean isDir(Path path) {
        return getAbs(path).toFile().isDirectory();
    }

    @Override
    public int run(String cmd, Consumer<String> stdOut, Consumer<String> stdErr) {
        String pwd = term.eval("pwd");
        try {
            term.eval("cd " + Bash.encode(getAbs(Path.of(""))));
            return term.exec(cmd, stdOut, stdErr);
        } finally {
            term.eval("cd " + pwd);
        }
    }

    @Override
    public void find(Function<Path, Boolean> needContinue, Consumer<Path> sayBye, Comparator<Path> sorting) {
        Path root = getRoot();
        FileSystem.tVerse(
                root,
                (p) -> needContinue.apply(root.relativize(p)),
                (p) -> sayBye.accept(root.relativize(p)),
                sorting
        );
    }

    @Override
    public void flush() {
        if (isReadOnly) {
            return;
        }

        if (!filesToAdd.toString().isBlank()) {
            String pwd = term.eval("pwd");
            try {
                term.eval("cd " + Bash.encode(getRoot()));
                Arrays.stream(filesToAdd.toString().split(System.lineSeparator()))
                        .forEach(cFile -> GitBindings.add(term, Path.of(cFile)));
                if (GitBindings.hasStatus(term)) {
                    GitBindings.commit(term);
                    term.eval("cd ..");
                    source.flush();
                } else {
                    log.debug("Status has no changes, skip commit.");
                }
            } finally {
                term.eval("cd " + pwd);
            }
        }

        filesToAdd = new StringBuffer();
    }

    @Override
    public FileSystem cd(Path jump) {
        return new FSOffsetProxy(this, jump);
    }


    @Override
    public void close() throws Exception {
        source.close();
    }


    public Path getRoot() {
        return source.getRoot();
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) {
            return true;
        }
        if (o == null || getClass() != o.getClass()) {
            return false;
        }
        VersionedFiles gitFiles = (VersionedFiles) o;
        return Objects.equals(source, gitFiles.source);
    }

    @Override
    public int hashCode() {
        return Objects.hash(source);
    }
}
